package bcn.serializer;

import bcn.lang.BehaviorDef;
import bcn.lang.Connection;
import bcn.lang.Controller;
import bcn.lang.InfluenceDef;
import bcn.lang.InputPortDef;
import bcn.lang.Instance;
import bcn.lang.JavaParam;
import bcn.lang.LangPackage;
import bcn.lang.PortDef;
import bcn.lang.PortRef;
import bcn.services.LangGrammarAccess;
import com.google.inject.Inject;
import com.google.inject.Provider;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.serializer.acceptor.ISemanticSequenceAcceptor;
import org.eclipse.xtext.serializer.acceptor.SequenceFeeder;
import org.eclipse.xtext.serializer.diagnostic.ISemanticSequencerDiagnosticProvider;
import org.eclipse.xtext.serializer.diagnostic.ISerializationDiagnostic.Acceptor;
import org.eclipse.xtext.serializer.sequencer.AbstractDelegatingSemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.GenericSequencer;
import org.eclipse.xtext.serializer.sequencer.ISemanticNodeProvider.INodesForEObjectProvider;
import org.eclipse.xtext.serializer.sequencer.ISemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService.ValueTransient;

@SuppressWarnings("all")
public class LangSemanticSequencer extends AbstractDelegatingSemanticSequencer {

	@Inject
	private LangGrammarAccess grammarAccess;
	
	public void createSequence(EObject context, EObject semanticObject) {
		if(semanticObject.eClass().getEPackage() == LangPackage.eINSTANCE) switch(semanticObject.eClass().getClassifierID()) {
			case LangPackage.BEHAVIOR_DEF:
				if(context == grammarAccess.getBehaviorDefRule() ||
				   context == grammarAccess.getDeclarationRule()) {
					sequence_BehaviorDef(context, (BehaviorDef) semanticObject); 
					return; 
				}
				else break;
			case LangPackage.CONNECTION:
				if(context == grammarAccess.getConnectionRule() ||
				   context == grammarAccess.getDeclarationRule()) {
					sequence_Connection(context, (Connection) semanticObject); 
					return; 
				}
				else break;
			case LangPackage.CONTROLLER:
				if(context == grammarAccess.getControllerRule()) {
					sequence_Controller(context, (Controller) semanticObject); 
					return; 
				}
				else break;
			case LangPackage.INFLUENCE_DEF:
				if(context == grammarAccess.getInfluenceDefRule()) {
					sequence_InfluenceDef(context, (InfluenceDef) semanticObject); 
					return; 
				}
				else break;
			case LangPackage.INPUT_PORT_DEF:
				if(context == grammarAccess.getInputPortDefRule()) {
					sequence_InputPortDef(context, (InputPortDef) semanticObject); 
					return; 
				}
				else break;
			case LangPackage.INSTANCE:
				if(context == grammarAccess.getDeclarationRule() ||
				   context == grammarAccess.getInstanceRule()) {
					sequence_Instance(context, (Instance) semanticObject); 
					return; 
				}
				else break;
			case LangPackage.JAVA_PARAM:
				if(context == grammarAccess.getJavaParamRule()) {
					sequence_JavaParam(context, (JavaParam) semanticObject); 
					return; 
				}
				else break;
			case LangPackage.PORT_DEF:
				if(context == grammarAccess.getPortDefRule()) {
					sequence_PortDef(context, (PortDef) semanticObject); 
					return; 
				}
				else break;
			case LangPackage.PORT_REF:
				if(context == grammarAccess.getPortRefRule()) {
					sequence_PortRef(context, (PortRef) semanticObject); 
					return; 
				}
				else break;
			}
		if (errorAcceptor != null) errorAcceptor.accept(diagnosticProvider.createInvalidContextOrTypeDiagnostic(semanticObject, context));
	}
	
	/**
	 * Constraint:
	 *     (name=ID types=JAVA_GENERIC_TYPES? (params+=JavaParam? params+=JavaParam*)? inputs+=InputPortDef* outputs+=PortDef*)
	 */
	protected void sequence_BehaviorDef(EObject context, BehaviorDef semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (src=PortRef dests+=PortRef dests+=PortRef*)
	 */
	protected void sequence_Connection(EObject context, Connection semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (pckg=FullID name=ID decls+=Declaration*)
	 */
	protected void sequence_Controller(EObject context, Controller semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (name=ID array=JavaIntExpr? min=JavaFloatExpr max=JavaFloatExpr dec=JavaFloatExpr)
	 */
	protected void sequence_InfluenceDef(EObject context, InfluenceDef semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (influence=InfluenceDef | (port=PortDef buffer=JavaIntExpr?))
	 */
	protected void sequence_InputPortDef(EObject context, InputPortDef semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (behavior=ID types=JAVA_GENERIC_TYPES? name=ID)
	 */
	protected void sequence_Instance(EObject context, Instance semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (type=JavaType name=ID)
	 */
	protected void sequence_JavaParam(EObject context, JavaParam semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, LangPackage.Literals.JAVA_PARAM__TYPE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, LangPackage.Literals.JAVA_PARAM__TYPE));
			if(transientValues.isValueTransient(semanticObject, LangPackage.Literals.JAVA_PARAM__NAME) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, LangPackage.Literals.JAVA_PARAM__NAME));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getJavaParamAccess().getTypeJavaTypeParserRuleCall_0_0(), semanticObject.getType());
		feeder.accept(grammarAccess.getJavaParamAccess().getNameIDTerminalRuleCall_1_0(), semanticObject.getName());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (type=JavaType name=ID array=JavaIntExpr?)
	 */
	protected void sequence_PortDef(EObject context, PortDef semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (behavior=[Instance|ID] port=ID index=JavaIntExpr?)
	 */
	protected void sequence_PortRef(EObject context, PortRef semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
}
