package foraging;

import com.cyberbotics.webots.controller.Compass;
import com.cyberbotics.webots.controller.GPS;


public class Attract extends foraging_stub.Attract_stub {
	
	static private final float EPSILON = 0.03f;  // 3 centimeters
	
	private final GPS gps;
	private final Compass compass;
	private Location target;
	
	
    public Attract(GPS gps, Compass compass) {
        super();
        this.gps = gps;
        this.compass = compass;
        target = null;
	}


	@Override
    public void step() {
        Location loc = _attractor.get();
        if (loc != null) {
            target = loc;  // consider the new target location
        }
        if (target != null) {  // new or previous location (from the last control step)
            // try to reach the target location
            double[] pos = gps.getValues();
            double[] north = compass.getValues();
            float heading = (float)Math.atan2(north[0], north[2]);
            float dx = (float)pos[0] - target.x;
            float dz = (float)pos[2] - target.z;
            if ((dx*dx + dz*dz) < EPSILON*EPSILON) {  // (almost) at the target location -> stop
                _drive.send(new Drive.Speed(0, 0));
            } else {  // move toward the target location
                float direction = Misc.stdAngle((float)Math.atan2(dz, dx) - heading);
                //System.out.println("Attract: direction = " + direction);
                if (Math.abs(direction) > (Math.PI-0.01)) {  // the target is (almost) behind -> always turn in the same direction
                    //System.out.println("Attract: sharp turn"); 
                    _drive.send(new Drive.Speed(1, -1));
                } else {
                    _drive.send(new Drive.Speed((direction >= 0  ?  1  :  (1 + 2*direction/(float)Math.PI)),
                            (direction <= 0  ?  1  :  (1 - 2*direction/(float)Math.PI))));
                }
            }
        }
        if (loc == null) {  // don't consider the old target any more
            target = null;
        }
    }
	
}
