package foraging;

import com.cyberbotics.webots.controller.Compass;
import com.cyberbotics.webots.controller.GPS;


public class Repulse extends foraging_stub.Repulse_stub {
    
    private final GPS gps;
    private final Compass compass;
    private Location source;
    
    public Repulse(GPS gps, Compass compass) {
        super();
        this.gps = gps;
        this.compass = compass;
        source = null;
    }
    
    
    @Override
    public void step() {
        Location loc = _repulsor.get();
        if (loc != null) {
            source = loc;  // consider the new source location
        }
        if (source != null) {  // new or previous location (from the last control step)
            // escape from the source location
            double[] pos = gps.getValues();
            double[] north = compass.getValues();
            float heading = (float)Math.atan2(north[0], north[2]);
            float dx = (float)pos[0] - source.x;
            float dz = (float)pos[2] - source.z;
            float direction = Misc.stdAngle((float)Math.PI + (float)Math.atan2(dz, dx) - heading);
            //System.out.println("Repulse: direction = " + direction);
            if (Math.abs(direction) > (Math.PI-0.01)) {  // the source is (almost) ahead -> always turn in the same direction
                //System.out.println("Repulse: sharp turn");
                _drive.send(new Drive.Speed(1, -1));
            } else {
                _drive.send(new Drive.Speed((direction >= 0  ?  1  :  (1 + 2*direction/(float)Math.PI)),
                                            (direction <= 0  ?  1  :  (1 - 2*direction/(float)Math.PI))));
            }
        }
        if (loc == null) {  // don't consider the old source any more
            source = null;
        }
    }
    
}
